package com.zipcodeconway;

public class ConwayGameOfLife {

    SimpleWindow window;
    private final int dimension;

    int[][] theBoard;// board represents the grid in 2d format
    int[][] next;

    public ConwayGameOfLife(Integer dimension) {
        this.dimension = dimension;
        this.theBoard = createRandomStart(dimension);
        this.next = new int[dimension][dimension];
        this.window = new SimpleWindow(dimension);
    }

    public ConwayGameOfLife(Integer dimension, int[][] startmatrix) {
        this.dimension = dimension;
        this.theBoard = startmatrix;
        this.next = new int[dimension][dimension];
        this.window = new SimpleWindow(dimension);
    }

    public static void main(String[] args) {
        ConwayGameOfLife sim = new ConwayGameOfLife(50);
        int[][] endingWorld = sim.simulate(2000);
    }

    // Contains the logic for the starting scenario.
    // Which cells are alive or dead in generation 0.
    // allocates and returns the starting matrix of size 'dimension'
    private int[][] createRandomStart(Integer dimension) {

        int[][] randoBoard = new int[dimension][dimension];
        for (int i = 0; i < (dimension * dimension)*.5; i++) {

            int x = (int) (Math.random() * dimension);

            int y = (int) (Math.random() * dimension);

            randoBoard[x][y] = 1;

        }

        return randoBoard;
    }

    public int[][] simulate(Integer maxGenerations) {
        for (int generation = 0; generation <= maxGenerations; generation++) {
            window.display(theBoard, generation);
            iterate();
            copyAndZeroOut(next, theBoard);
            window.sleep(100);
        }

        return theBoard;
    }

    // copy the values of 'next' matrix to 'current' matrix,
    // and then zero out the contents of 'next' matrix
    public void copyAndZeroOut(int[][] next, int[][] current) {
        for(int i = 0; i < dimension; i++){
            for(int j = 0; j < dimension; j++){
                current[i][j] = next[i][j];
                next[i][j] = 0;
            }
        }
    }

    // Calculate if an individual cell should be alive in the next generation.
    // Based on the game logic:
	/*
		Any live cell with fewer than two live neighbours dies, as if by needs caused by underpopulation.
		Any live cell with more than three live neighbours dies, as if by overcrowding.
		Any live cell with two or three live neighbours lives, unchanged, to the next generation.
		Any dead cell with exactly three live neighbours cells will come to life.
	*/
    private int isAlive(int row /*y axis*/, int col/*x axis*/, int[][] board) {
        int count = 0;
        int returnValue = 0;
        int x;
        int y;

//        int pointOfInterest = board[col][row];

        for (int i = row - 1; i <= row + 1; i++) {
            for (int j = col - 1; j <= col + 1; j++) {

                x = i;
                y = j;

                if (i == -1) {
                    i = dimension - 1;
                }

                if (i == dimension) {
                    i = 0;
                }

                if (j == -1) {
                    j = dimension - 1;
                }

                if (j == dimension) {
                    j = 0;
                }

                if (board[i][j] == 1 && (!(i == row && j == col))) {
                    count++;
                }

                i = x;
                j = y;

            }
        }

            if (count == 3) {
                returnValue = 1;
            }
            if (board[row][col] == 1 && count == 2){
                returnValue = 1;
            }

            return returnValue;
        }




    public void iterate() {
        for (int i = 0; i < theBoard.length; i++) {
            for (int j = 0; j < theBoard.length; j++) {
                next[i][j] = isAlive(i, j, theBoard);
            }
        }
    }




}
